package cn.com.qmth.examcloud.tool.service.export_student_answer_and_score_detail;

import cn.com.qmth.examcloud.tool.config.SysProperty;
import cn.com.qmth.examcloud.tool.entity.TaskEntity;
import cn.com.qmth.examcloud.tool.service.CommonService;
import cn.com.qmth.examcloud.tool.utils.DateHelper;
import cn.com.qmth.examcloud.tool.utils.FileHelper;
import cn.com.qmth.examcloud.tool.utils.JsonMapper;
import cn.com.qmth.examcloud.tool.utils.StatusException;
import cn.com.qmth.examcloud.tool.vo.User;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.handler.SheetWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteWorkbookHolder;
import com.fasterxml.jackson.databind.JsonNode;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;

@Component
public class ExportStudentAnswerAndScoreDetailTask {

    private final static Logger log = LoggerFactory.getLogger(ExportStudentAnswerAndScoreDetailTask.class);

    @Autowired
    private CommonService commonService;

    @Autowired
    private SysProperty sysProperty;

    public void start(TaskEntity task) {
        JsonNode jsonParams = new JsonMapper().getNode(task.getParams());
        if (jsonParams == null) {
            throw new StatusException("任务参数解析错误！");
        }

        final String tempDir = "export_" + DateHelper.currentTimeMillis() + "_" + task.getId();

        User user = commonService.login(sysProperty.getServerDomain(), sysProperty.getServerLoginName(), sysProperty.getServerPassword());
        this.execute(jsonParams, user, tempDir);

        // 压缩文件
        FileHelper.zip(new File(sysProperty.getDataDir() + "/" + tempDir),
                new File(sysProperty.getDataDir() + "/" + tempDir + ".zip"));
        task.setFilePath(tempDir + ".zip");
    }

    private void execute(JsonNode jsonParams, User user, String tempDir) {
        Long examId = jsonParams.get("examId").asLong(0L);
        String courseCodes = jsonParams.get("courseCodes").asText("");
        Boolean needSubjective = jsonParams.get("needSubjective").asBoolean(false);
        log.info("examId = {}, courseCodes = {}, needSubjective = {}", examId, courseCodes, needSubjective);

        if (StringUtils.isNotBlank(courseCodes)) {
            String[] courseCodeList = StringUtils.split(courseCodes, ",");

            for (String courseCode : courseCodeList) {
                //todo

                final String dataFilePath = tempDir + "/" + examId + "_" + courseCode + ".xlsx";
                File dataFile = new File(sysProperty.getDataDir() + "/" + dataFilePath);
                FileHelper.makeDirs(dataFile.getParent());

                EasyExcel.write(dataFile, User.class).registerWriteHandler(new SheetWriteHandler() {
                    @Override
                    public void afterSheetCreate(WriteWorkbookHolder workbookHolder, WriteSheetHolder sheetHolder) {
                        Sheet sheet = sheetHolder.getSheet();
                        sheet.createFreezePane(1, 1);
                    }
                }).sheet().doWrite(new ArrayList<>());
            }
        }
    }

}
